require 'slack-ruby-client'
require 'eventmachine'
require 'faye/websocket'

require 'pepito/source'
require 'pepito/adapter'
require 'pepito/message'

module Pepito
  module Adapters
    # Adapter for slack
    class Slack < Adapter
      class << self
        # Configs needed for Slack
        # @return [Array<Hash>]
        def configs
          [
            {
              name: 'SLACK_API_TOKEN',
              required: true
            }
          ]
        end
      end

      # Slack RealTime client
      # @return [Slack::RealTime::Client]
      attr_reader :client

      def initialize(robot, config)
        super(robot, config)

        configure_slack_api

        @client = ::Slack::RealTime::Client.new
        @client.web_client.auth_test

        slack_on_message
      end

      # Runs the slack client
      # @return [void]
      def run
        @client.start!
      end

      # Send messages to slack.
      # @param target [Pepito::Source] Where to send the message to.
      # @param strings [Array<String>] Strings to output.
      # @return [void]
      def send_messages(target, strings)
        message = ''
        strings.each do |s|
          message = message + s + "\n"
        end
        @client.message(channel: target.room, text: message)
      end

      # Set topic
      # @param topic [String] new topic string
      # @return [void]
      def topic(target, topic)
        @client.web_client.channels_setTopic(channel: target.room, topic: topic)
      end

      # Stops the client
      # @return [void]
      def stop
        @client.stop!
      end

      private

      # Configures the slack API
      # @return [void]
      def configure_slack_api
        ::Slack.configure do |c|
          c.token = @config['SLACK_API_TOKEN']
        end

        ::Slack::RealTime.configure do |c|
          c.concurrency = ::Slack::RealTime::Concurrency::Eventmachine
        end
      end

      # Behavior for slack when it receives a message
      # @return [void]
      def slack_on_message
        @client.on :message do |data|
          unless data['reply_to']
            source = Source.new(self, data['user'], data['channel'], false)
            @robot.receive_message(Message.new(@robot, source, data['text']))
          end
        end
      end
    end
  end
end
